﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Linq;
using System.Windows.Forms;
using Telerik.WinControls;
using Telerik.WinControls.UI;

namespace DragAndDropGridViewToGridView
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
            leftGrid.ShowGroupPanel = false;
            rightGrid.ShowGroupPanel = false;
            leftGrid.AllowAddNewRow = false;
            rightGrid.AllowAddNewRow = false;
            leftGrid.AutoSizeColumnsMode = GridViewAutoSizeColumnsMode.Fill;
            rightGrid.AutoSizeColumnsMode = GridViewAutoSizeColumnsMode.Fill;

            radListView1.AllowDragDrop = true;
            radListView1.ListViewElement.DragDropService.PreviewDragOver += new EventHandler<Telerik.WinControls.RadDragOverEventArgs>(DragDropService_PreviewDragOver);
            radListView1.ListViewElement.DragDropService.PreviewDragDrop += new EventHandler<Telerik.WinControls.RadDropEventArgs>(DragDropService_PreviewDragDrop);
        }

        void DragDropService_PreviewDragDrop(object sender, Telerik.WinControls.RadDropEventArgs e)
        {
            var listviewVisualItem = e.DragInstance as SimpleListViewVisualItem;

            if (listviewVisualItem == null)
            {
                return;
            }
            e.Handled = true;

            var dropTarget = e.HitTarget as RadItem;
            var targetGrid = dropTarget.ElementTree.Control as RadGridView;
            if (targetGrid == null)
            {
                return;
            }

            var dragListView = listviewVisualItem.ElementTree.Control as RadListView;

            //append dragged rows to the end of the target grid
            int index = targetGrid.RowCount;

            //Grab every selected row from the source grid, including the current row
            List<ListViewDataItem> items = dragListView.SelectedItems.ToList<ListViewDataItem>();
            if (dragListView.CurrentItem != null)
            {
                ListViewDataItem item = dragListView.CurrentItem;
                if (!items.Contains(item))
                    items.Add(item);
            }

            this.MoveRows(targetGrid, dragListView, items, index);
        }

        private void MoveRows(RadGridView targetGrid, RadListView dragListView, IList<ListViewDataItem> dragitems, int index)
        {
            dragListView.BeginUpdate();
            targetGrid.BeginUpdate();
            for (int i = dragitems.Count - 1; i >= 0; i--)
            {
                ListViewDataItem item = dragitems[i];

                if (targetGrid.DataSource == null)
                {
                    //unbound scenario
                    GridViewRowInfo newRow = targetGrid.Rows.NewRow();
                    newRow.Cells[0].Value = item.Value;

                    targetGrid.Rows.Insert(index, newRow);

                    dragListView.Select(new ListViewDataItem[] { item});
                    dragListView.Items.Remove(item);
                }
                else if (typeof(DataSet).IsAssignableFrom(targetGrid.DataSource.GetType()))
                {
                    //bound to a dataset scenario
                    var sourceTable = ((DataSet)dragListView.DataSource).Tables[0];
                    var targetTable = ((DataSet)targetGrid.DataSource).Tables[0];

                    var newRow = targetTable.NewRow();
                    newRow[0] = item.Value;

                    sourceTable.Rows.Remove(((DataRowView)item.DataBoundItem).Row);
                    targetTable.Rows.InsertAt(newRow, index);
                }
                else if (typeof(IList).IsAssignableFrom(targetGrid.DataSource.GetType()))
                {
                    //bound to a list of objects scenario
                    var targetCollection = (IList)targetGrid.DataSource;
                    var sourceCollection = (IList)dragListView.DataSource;
                    sourceCollection.Remove(item.DataBoundItem);
                    targetCollection.Add(item.DataBoundItem);
                }
                else
                {
                    throw new ApplicationException("Unhandled Scenario");
                }
                index++;
            }
            dragListView.EndUpdate();
            targetGrid.EndUpdate(true);
        }


        void DragDropService_PreviewDragOver(object sender, Telerik.WinControls.RadDragOverEventArgs e)
        {
            if (e.DragInstance is SimpleListViewVisualItem)
            {
                e.CanDrop = e.HitTarget is GridDataRowElement ||
                            e.HitTarget is GridTableElement ||
                            e.HitTarget is GridSummaryRowElement;
            }
        }

        private void ResetGrids()
        {
            leftGrid.DataSource = null;
            leftGrid.Rows.Clear();
            leftGrid.Columns.Clear();
            rightGrid.DataSource = null;
            rightGrid.Rows.Clear();
            rightGrid.Columns.Clear();
        }

        private void btnUnbound_Click(object sender, EventArgs e)
        {
            ResetGrids();

            PrepareUnboundGrid(leftGrid);
            leftGrid.Rows.Add("Carey", "Payette");
            leftGrid.Rows.Add("Michael", "Crump");
            leftGrid.Rows.Add("Jeff", "Fritz");
            PrepareUnboundGrid(rightGrid);
            rightGrid.Rows.Add("Phil", "Japikse");
            rightGrid.Rows.Add("Jesse", "Liberty");
            rightGrid.Rows.Add("Iris", "Classon");
        }

        private void PrepareUnboundGrid(RadGridView grid)
        {
            //setup columns
            GridViewTextBoxColumn firstName = new GridViewTextBoxColumn("FirstName", "FirstName");
            firstName.HeaderText = "First Name";
            GridViewTextBoxColumn lastName = new GridViewTextBoxColumn("LastName", "LastName");
            lastName.HeaderText = "Last Name";

            grid.Columns.AddRange(firstName, lastName);
        }

        private void btnBoundObjects_Click(object sender, EventArgs e)
        {
            ResetGrids();

            BindingList<Player> dataList1 = new BindingList<Player>();
            dataList1.Add(new Player() { FirstName = "Carey", LastName = "Payette" });
            dataList1.Add(new Player() { FirstName = "Michael", LastName = "Crump" });
            dataList1.Add(new Player() { FirstName = "Jeff", LastName = "Fritz" });
            BindingList<Player> dataList2 = new BindingList<Player>();
            dataList2.Add(new Player() { FirstName = "Phil", LastName = "Japikse" });
            dataList2.Add(new Player() { FirstName = "Jesse", LastName = "Liberty" });
            dataList2.Add(new Player() { FirstName = "Iris", LastName = "Classon" });

            leftGrid.DataSource = dataList1;
            rightGrid.DataSource = dataList2;
        }

        private void btnBoundDataSet_Click(object sender, EventArgs e)
        {
            ResetGrids();

            DataSet ds1 = new DataSet();
            DataTable team1 = new DataTable();
            team1.Columns.Add("First Name", typeof(string));
            team1.Columns.Add("Last Name", typeof(string));
            team1.Rows.Add("Carey", "Payette");
            team1.Rows.Add("Michael", "Crump");
            team1.Rows.Add("Jeff", "Fritz");
            ds1.Tables.Add(team1);

            DataSet ds2 = new DataSet();
            DataTable team2 = new DataTable();
            team2.Columns.Add("First Name", typeof(string));
            team2.Columns.Add("Last Name", typeof(string));
            team2.Rows.Add("Phil", "Japikse");
            team2.Rows.Add("Jesse", "Liberty");
            team2.Rows.Add("Iris", "Classon");
            ds2.Tables.Add(team2);

            leftGrid.DataSource = ds1;
            leftGrid.DataMember = "Table1";
            rightGrid.DataSource = ds2;
            rightGrid.DataMember = "Table1";
        }
    }

    public class Player
    {
        public string FirstName { get; set; }
        public string LastName { get; set; }
    }
}
